@@ -2,7 +2,7 @@
  * matroska_segment.hpp : matroska demuxer
  *****************************************************************************
  * Copyright (C) 2003-2004 VLC authors and VideoLAN
- * $Id: afb37dbc9f274e53e438de3c15441d8466c0e6b2 $
+ * $Id: c753644cc8794c7e5a6c418cb797e1e069bebfdf $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *          Steve Lhomme <steve.lhomme@free.fr>
@@ -22,10 +22,19 @@
  * Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
-#ifndef _MATROSKA_SEGMENT_HPP_
-#define _MATROSKA_SEGMENT_HPP_
+#ifndef VLC_MKV_MATROSKA_SEGMENT_HPP_
+#define VLC_MKV_MATROSKA_SEGMENT_HPP_
 
 #include "mkv.hpp"
+#include "matroska_segment_seeker.hpp"
+#include <vector>
+#include <string>
+
+#include <map>
+#include <set>
+#include <memory>
+
+#include "Ebml_parser.hpp"
 
 class EbmlParser;
 
@@ -33,8 +42,7 @@ class chapter_edition_c;
 class chapter_translation_c;
 class chapter_item_c;
 
-struct mkv_track_t;
-struct mkv_index_t;
+class mkv_track_t;
 
 typedef enum
 {
@@ -48,31 +56,31 @@ typedef enum
 class SimpleTag
 {
 public:
-    SimpleTag():
-        psz_tag_name(NULL), psz_lang(NULL), b_default(true), p_value(NULL){}
-    ~SimpleTag();
-    char *psz_tag_name;
-    char *psz_lang; /* NULL value means "undf" */
-    bool b_default;
-    char * p_value;
-    std::vector<SimpleTag *> sub_tags;
+    typedef std::vector<SimpleTag> sub_tags_t;
+    std::string tag_name;
+    std::string lang;
+    std::string value;
+    sub_tags_t sub_tags;
 };
 
 class Tag
 {
 public:
+    typedef std::vector<SimpleTag> simple_tags_t;
     Tag():i_tag_type(WHOLE_SEGMENT),i_target_type(50),i_uid(0){}
-    ~Tag();
     tag_target_type i_tag_type;
     uint64_t        i_target_type;
     uint64_t        i_uid;
-    std::vector<SimpleTag*> simple_tags;
+    simple_tags_t   simple_tags;
 };
 
 class matroska_segment_c
 {
 public:
-    matroska_segment_c( demux_sys_t & demuxer, EbmlStream & estream );
+    typedef std::map<mkv_track_t::track_id_t, std::unique_ptr<mkv_track_t>> tracks_map_t;
+    typedef std::vector<Tag>            tags_t;
+
+    matroska_segment_c( demux_sys_t &, EbmlStream &, KaxSegment * );
     virtual ~matroska_segment_c();
 
     KaxSegment              *segment;
@@ -83,10 +91,11 @@ public:
 
     /* duration of the segment */
     mtime_t                 i_duration;
-    mtime_t                 i_start_time;
+    mtime_t                 i_mk_start_time;
 
     /* all tracks */
-    std::vector<mkv_track_t*> tracks;
+    tracks_map_t tracks;
+    SegmentSeeker::track_ids_t priority_tracks;
 
     /* from seekhead */
     int                     i_seekhead_count;
@@ -95,21 +104,15 @@ public:
     int64_t                 i_tracks_position;
     int64_t                 i_info_position;
     int64_t                 i_chapters_position;
-    int64_t                 i_tags_position;
     int64_t                 i_attachments_position;
 
     KaxCluster              *cluster;
     uint64                  i_block_pos;
-    uint64                  i_cluster_pos;
-    int64_t                 i_start_pos;
     KaxSegmentUID           *p_segment_uid;
     KaxPrevUID              *p_prev_segment_uid;
     KaxNextUID              *p_next_segment_uid;
 
     bool                    b_cues;
-    int                     i_index;
-    int                     i_index_max;
-    mkv_index_t             *p_indexes;
 
     /* info */
     char                    *psz_muxing_application;
@@ -122,31 +125,36 @@ public:
     /* when you remove this variable the compiler issues an atomicity error */
     /* this variable only works when using std::vector<chapter_edition_c> */
     std::vector<chapter_edition_c*> stored_editions;
-    int                             i_default_edition;
+    std::vector<chapter_edition_c*>::size_type i_default_edition;
 
     std::vector<chapter_translation_c*> translations;
     std::vector<KaxSegmentFamily*>  families;
-    std::vector<Tag *>              tags;
+    tags_t                          tags;
 
     demux_sys_t                    & sys;
-    EbmlParser                     *ep;
+    EbmlParser                     ep;
     bool                           b_preloaded;
     bool                           b_ref_external_segments;
 
     bool Preload();
     bool PreloadFamily( const matroska_segment_c & segment );
+    bool PreloadClusters( uint64 i_cluster_position );
     void InformationCreate();
-    void Seek( mtime_t i_date, mtime_t i_time_offset, int64_t i_global_position );
+
+    bool FastSeek( demux_t &, mtime_t i_mk_date, mtime_t i_mk_time_offset );
+    bool Seek( demux_t &, mtime_t i_mk_date, mtime_t i_mk_time_offset );
+
     int BlockGet( KaxBlock * &, KaxSimpleBlock * &, bool *, bool *, int64_t *);
 
-    int BlockFindTrackIndex( size_t *pi_track,
-                             const KaxBlock *, const KaxSimpleBlock * );
+    mkv_track_t * FindTrackByBlock(const KaxBlock *, const KaxSimpleBlock * );
 
-    bool Select( mtime_t i_start_time );
-    void UnSelect();
+    bool ESCreate( );
+    void ESDestroy( );
 
     static bool CompareSegmentUIDs( const matroska_segment_c * item_a, const matroska_segment_c * item_b );
 
+    bool SameFamily( const matroska_segment_c & of_segment ) const;
+
 private:
     void LoadCues( KaxCues *cues );
     void LoadTags( KaxTags *tags );
@@ -157,12 +165,18 @@ private:
     void ParseSeekHead( KaxSeekHead *seekhead );
     void ParseTracks( KaxTracks *tracks );
     void ParseChapterAtom( int i_level, KaxChapterAtom *ca, chapter_item_c & chapters );
-    void ParseTrackEntry( KaxTrackEntry *m );
-    void ParseCluster( bool b_update_start_time = true );
-    SimpleTag * ParseSimpleTags( KaxTagSimple *tag, int level = 50 );
+    void ParseTrackEntry( const KaxTrackEntry* m );
+    bool ParseCluster( KaxCluster *cluster, bool b_update_start_time = true, ScopeMode read_fully = SCOPE_ALL_DATA );
+    bool ParseSimpleTags( SimpleTag* out, KaxTagSimple *tag, int level = 50 );
     void IndexAppendCluster( KaxCluster *cluster );
-    int32_t TrackInit( mkv_track_t * p_tk );
+    bool TrackInit( mkv_track_t * p_tk );
     void ComputeTrackPriority();
+    void EnsureDuration();
+    bool InternalSeek( demux_t &, mtime_t i_mk_date, mtime_t i_mk_time_offset, bool b_accurate );
+
+    SegmentSeeker _seeker;
+
+    friend SegmentSeeker;
 };
 
 
