@@ -2,7 +2,7 @@
  * nsc.c: NSC file demux and encoding decoder
  *****************************************************************************
  * Copyright (C) 2005 VLC authors and VideoLAN
- * $Id: 719c59dc36d012c8dd3dcaebac4b567f9aedbd46 $
+ * $Id: 068bb4f8257b14ce1755e7d15c6820ad7d70c337 $
  *
  * Authors: Derk-Jan Hartman <hartman at videolan dot org>
  *          based on work from Jon Lech Johansen <jon@nanocrew.net>
@@ -237,7 +237,7 @@ static int DemuxOpen( vlc_object_t * p_this )
     int i_size;
 
     /* Lets check the content to see if this is a NSC file */
-    i_size = stream_Peek( p_demux->s, &p_peek, MAX_LINE );
+    i_size = vlc_stream_Peek( p_demux->s, &p_peek, MAX_LINE );
     i_size -= sizeof("NSC Format Version=") - 1;
 
     if ( i_size > 0 )
@@ -322,7 +322,7 @@ static int Demux ( demux_t *p_demux )
 {
     char            *psz_line;
 
-    while( ( psz_line = stream_ReadLine( p_demux->s ) ) )
+    while( ( psz_line = vlc_stream_ReadLine( p_demux->s ) ) )
     {
         ParseLine( p_demux, psz_line );
         free( psz_line );
