@@ -2,7 +2,7 @@
  * folder.c
  *****************************************************************************
  * Copyright (C) 2006 VLC authors and VideoLAN
- * $Id: 38c1e075f75b5fc96b065c7cd4c97381fe7b6ed8 $
+ * $Id: d243172973eef9219e11ee07b7f1951cacfbb617 $
  *
  * Authors: Antoine Cellerier <dionoea -at- videolan -dot- org>
  *
@@ -45,6 +45,12 @@ static const char* cover_files[] = {
     "Album.jpg",
     ".folder.png",          /* KDE?    */
     "cover.jpg",            /* rockbox */
+    "cover.png",
+    "cover.gif",
+    "front.jpg",
+    "front.png",
+    "front.gif",
+    "front.bmp",
     "thumb.jpg",
 };
 
@@ -92,7 +98,7 @@ static int FindMeta( vlc_object_t *p_this )
             free( psz_uri );
             return VLC_EGENERIC;
         }
-        char *psz_basedir = make_path( psz_path );
+        char *psz_basedir = vlc_uri2path( psz_path );
         FREENULL( psz_path );
         if( psz_basedir == NULL )
         {
@@ -107,7 +113,7 @@ static int FindMeta( vlc_object_t *p_this )
 
     if ( psz_path == NULL )
     {
-        char *psz_basedir = make_path( psz_uri );
+        char *psz_basedir = vlc_uri2path( psz_uri );
         if( psz_basedir == NULL )
         {
             free( psz_uri );
