@@ -2,7 +2,7 @@
  * stream_output.h : internal stream output
  *****************************************************************************
  * Copyright (C) 2002-2005 VLC authors and VideoLAN
- * $Id: 4580c9279975c71eba985ce051969d2fd49dbf7b $
+ * $Id: c7ff95269d239e872205bc9740dcb6a71053006a $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *          Laurent Aimar <fenrir@via.ecp.fr>
@@ -37,8 +37,6 @@ struct sout_packetizer_input_t
 {
     sout_instance_t     *p_sout;
 
-    es_format_t         *p_fmt;
-
     sout_stream_id_sys_t    *id;
 };
 
@@ -46,26 +44,10 @@ sout_instance_t *sout_NewInstance( vlc_object_t *, const char * );
 #define sout_NewInstance(a,b) sout_NewInstance(VLC_OBJECT(a),b)
 void sout_DeleteInstance( sout_instance_t * );
 
-sout_packetizer_input_t *sout_InputNew( sout_instance_t *, es_format_t * );
+sout_packetizer_input_t *sout_InputNew( sout_instance_t *, const es_format_t * );
 int sout_InputDelete( sout_packetizer_input_t * );
 int sout_InputSendBuffer( sout_packetizer_input_t *, block_t* );
-
-/* Announce system */
-
-struct session_descriptor_t
-{
-    struct sockaddr_storage orig;
-    socklen_t origlen;
-    struct sockaddr_storage addr;
-    socklen_t addrlen;
-
-    char *psz_sdp;
-    bool b_ssm;
-};
-
-struct sap_handler_t *SAP_Create (vlc_object_t *);
-void SAP_Destroy (struct sap_handler_t *);
-int SAP_Add (struct sap_handler_t *, session_descriptor_t *);
-void SAP_Del (struct sap_handler_t *, const session_descriptor_t *);
+bool sout_InputIsEmpty(sout_packetizer_input_t *);
+void sout_InputFlush( sout_packetizer_input_t * );
 
 #endif
